import numpy as np
import sys
import matplotlib.pyplot as plt
import pysa.emd as emd
import pysa.utils as utils
import api.sensor_net_map as net_map
import util.file_handler as file_handler
import util.plotter_scroll as plotter_scroll
import constants


########################################################################################################################
#                                  Activity: SEGMENTATION and EMD-filtering                                            #
#                                                                                                                      #
#   This activity loads the EEG recordings (MAT-files) and segments the data: assigning the correct events to the      #
#   corresponding data segments and storing the data in a matrix. Matrix dimensions = [n_electrodes x data_length].    #
#   In addition to data segmentation, EMD-filtering is applied in order to remove line noise from each electrode.      #
#   (This could be its own activity, but it is always done after data segmentation.)                                   #
#                                                                                                                      #
########################################################################################################################

def start(username=''):
    # ------------------------------------------------USER INFORMATION------------------------------------------------ #
    if username == '':
        print "Please specify subject username in 'main.py'"
        sys.exit()

    # Should be 'True' if this is the first run, otherwise set to 'False' such that the segmented data matrix is loaded.
    do_segmentation = True
    # Should be 'True' if EMD is to be used to remove the first two IMFs (often contaminated by line noise)
    do_emd_filtering = True

    # Different usernames (folders) and the names of the exported MAT-files.
    if username == 'geir':
        filename = 'Geir 20160412 1428.mat'
        data_matrix_name = 'Geir_20160412_14282'

    elif username == 'eirik':
        filename = 'Eirik 20160413 1533.mat'
        data_matrix_name = 'Eirik_20160413_15332'

    elif username == 'daniel':
        filename = 'Daniel 20160414 1544.mat'
        data_matrix_name = 'Daniel_20160414_15442'

    elif username == 'thor':
        filename = 'Thor 20160415 1541.mat'
        data_matrix_name = 'Thor_20160415_15412'

    elif username == 'mats':
        filename = 'Mats 20160420 1518.mat'
        data_matrix_name = 'Mats_20160420_15182'

    elif username == 'kristine':
        filename = 'Kristine 20160421 1321.mat'
        data_matrix_name = 'Kristine_20160421_13212'

    elif username == 'fredrik':
        filename = 'FredrikW 20160421 1427.mat'
        data_matrix_name = 'FredrikW_20160421_14272'

    elif username == 'thomas':
        filename = 'Thomas 20160422 1306.mat'
        data_matrix_name = 'Thomas_20160422_13062'

    # Constant parameters (Specific to this experimental paradigm)
    samples_per_trial = 3000  # = 6sec
    stimuli_onset = 500       # = 1sec
    filename_seg = username + '_data_motor_cortex_segmented.mat'
    # filename_seg = username + '_data_256_segmented.mat'
    session_info_name = 'ECI_TCPIP_55513'

    # ---------------------------------------------------------------------------------------------------------------- #
    print("\x1b[2J\x1b[H")
    introduction = "EEG segmentation and EMD-filtering session - Subject: " + username.title()
    print "%s\n%s\n%s" % ('-' * len(introduction), introduction, '-' * len(introduction))

    # ------------------------------------------------DATA SEGMENTATION----------------------------------------------- #
    if do_segmentation:
        print 'Segmenting EEG data...'
        # Load .mat file to dictionary
        session_eeg_data = file_handler.read_mat_file_into_json_object(user=username, filename=filename)
        # Get sampling frequency
        sample_frequency = float(session_eeg_data['samplingRate'])  # [Hz] (500.0)
        data_matrix, events = net_map.get_segmented_data_matrix(session_eeg_data, session_info_name, data_matrix_name,
                                                                samples_per_trial=samples_per_trial,
                                                                stimuli_onset=stimuli_onset)
        print "Total number of electrodes: " + str(len(data_matrix)-1)
        data_matrix = data_matrix[constants.electrodes_motor_cortex]
        print "Number of electrodes at sensorimotor cortex: " + str(len(data_matrix))
        file_handler.save_segmented_data_as_mat(data_matrix, events, username, filename_seg)
        print "Segmentation done."

    else:
        # Load .mat file to dictionary
        data_json = file_handler.read_mat_file_into_json_object(username, filename_seg)
        data_matrix = data_json['data_seg']
        events = data_json['events']
        sample_frequency = 500.0
        print "Segmented EEG data loaded."
    # Get number of trials (specific to the experiment)
    n_trials = len(events)

    print 'Total number of trials: ' + str(n_trials)

    # ---------------------------------------------------------------------------------------------------------------- #

    # -----------------------------------------------EMD DYADIC FILTER------------------------------------------------ #
    data_json_filtered = file_handler.read_mat_file_into_json_object(username, filename_seg)
    data_matrix_filtered = data_json_filtered['data_seg']

    n_electrodes = len(data_matrix_filtered)
    # print 'Total number of electrodes: ' + str(n_electrodes)
    if do_emd_filtering:
        print 'Filtering using EMD...'
        line_noise_matrix = np.zeros(np.shape(data_matrix_filtered), np.float)
        trend_matrix = np.zeros(np.shape(data_matrix_filtered), np.float)
        # Loop through all the electrodes:
        for k in range(n_electrodes):
            # Loop through the entire data matrix, one trial at the time
            for j in range(0, n_trials*samples_per_trial, samples_per_trial):
                step = samples_per_trial
                data = data_matrix_filtered[k, j:j+step]
                min_data = min(data_matrix_filtered[k, j:j+step])
                max_data = max(data_matrix_filtered[k, j:j+step])
                # Run empirical mode decomposition (EMD)
                imfs = emd.emd(data, min_data, max_data, max_modes=10, max_siftings=400)
                # Store the last IMF (the trend) to remove slow drift in the data
                trend = imfs[-1]
                # Reverse normalize the trend and subtract the mean voltage
                trend_rev = utils.reverse_normalization(trend, min_data, max_data, samples_per_trial)
                trend_rev -= np.mean(trend_rev)
                trend_matrix[k, j:j+step] += trend_rev
                # Obtain the first two IMFs (the ones that are contaminated by line noise, 50Hz and 150Hz)
                imfs = imfs[:2]
                # Reverse normalize the IMFs and subtract the mean voltage
                for imf in range(len(imfs)):
                    imf_rev = utils.reverse_normalization(imfs[imf], min_data, max_data, samples_per_trial)
                    imf_rev -= np.mean(imf_rev)
                    line_noise_matrix[k, j:j+step] += imf_rev

            elec = k + 1
            if elec == int(0.1*n_electrodes):
                print 'EMD filtering 10% finished...'
            elif elec == int(0.2*n_electrodes):
                print 'EMD filtering 20% finished...'
            elif elec == int(0.3*n_electrodes):
                print 'EMD filtering 30% finished...'
            elif elec == int(0.4*n_electrodes):
                print 'EMD filtering 40% finished...'
            elif elec == int(0.5*n_electrodes):
                print 'EMD filtering 50% finished...'
            elif elec == int(0.6*n_electrodes):
                print 'EMD filtering 60% finished...'
            elif elec == int(0.7*n_electrodes):
                print 'EMD filtering 70% finished...'
            elif elec == int(0.8*n_electrodes):
                print 'EMD filtering 80% finished...'
            elif elec == int(0.9*n_electrodes):
                print 'EMD filtering 90% finished...'

        # Subtract the line noise and the trends in order to get the EMD-filtered data matrix:
        data_matrix_filtered -= line_noise_matrix
        data_matrix_filtered -= trend_matrix

        # Store
        print "EMD filtering done. Storing the data."

        file_handler.save_experiment_data_as_mat(data_matrix_filtered, 'data_matrix_emd_filtered', username,
                                                 'data_matrix_emd_filtered.mat')

        file_handler.save_experiment_data_as_mat(line_noise_matrix, 'line_noise_matrix', username,
                                                 'line_noise_matrix.mat')

        file_handler.save_experiment_data_as_mat(trend_matrix, 'trend_matrix', username, 'trend_matrix.mat')

    # ---------------------------------------------------------------------------------------------------------------- #
    # -------------------------------------------------PLOTS---------------------------------------------------------- #

    # Plots:
    plotter_scroll.plot_data_sliding(data_matrix_filtered, events, sample_frequency, title='Filtered data',
                                     n_sec_per_trial=6, n_trial_per_frame=2)
    plotter_scroll.plot_data_sliding(data_matrix, events, sample_frequency, title='Raw EEG data', n_sec_per_trial=6,
                                     n_trial_per_frame=2)
    plotter_scroll.plot_data_sliding(trend_matrix, events, sample_frequency, title='Trend matrix', n_sec_per_trial=6,
                                     n_trial_per_frame=2)

    try:
        plt.draw()
        plt.show(block=True)
    except KeyboardInterrupt, ki:
        sys.exit()
    # ---------------------------------------------------------------------------------------------------------------- #

