import numpy as np
import constants
from scipy import signal
import scipy.interpolate as interpolate


def get_segmented_data_matrix(session_eeg_data, session_info_name, data_matrix_name, samples_per_trial=3000, stimuli_onset=500):

    session_info_matrix = session_eeg_data[session_info_name]
    data_matrix = session_eeg_data[data_matrix_name]
    info_row = session_info_matrix[0, :]
    control_row = session_info_matrix[-2, :]
    sample_row = session_info_matrix[-1, :]
    segmented_data_matrix = np.ndarray(([]), np.float)
    first_trial = True
    events = []
    for i in range(len(info_row) - 1):

        if control_row[i] == 2:

            start_sample = int(int(sample_row[i]) - stimuli_onset)
            end_sample = int(start_sample + samples_per_trial)

            if info_row[i] == 'stmn':
                events.append('N')
                if first_trial:
                    first_trial = False
                    segmented_data_matrix = data_matrix[:, start_sample:end_sample]
                else:
                    segmented_data_matrix = np.hstack((segmented_data_matrix, data_matrix[:, start_sample:end_sample]))

            elif info_row[i] == 'stmr':
                events.append('R')
                if first_trial:
                    first_trial = False
                    segmented_data_matrix = data_matrix[:, start_sample:end_sample]
                else:
                    segmented_data_matrix = np.hstack((segmented_data_matrix, data_matrix[:, start_sample:end_sample]))

            elif info_row[i] == 'stml':
                events.append('L')
                if first_trial:
                    first_trial = False
                    segmented_data_matrix = data_matrix[:, start_sample:end_sample]
                else:
                    segmented_data_matrix = np.hstack((segmented_data_matrix, data_matrix[:, start_sample:end_sample]))

    return segmented_data_matrix, events


def map_from_motor_cortex_to_electrode(index):
    motor_cortex = constants.electrodes_motor_cortex
    electrode_index = motor_cortex[index]
    return electrode_index


def check_for_bad_electrodes(data_matrix, threshold=750.0, n_trials=170):

    data_matrix_checked = data_matrix

    num_electrodes = len(data_matrix_checked)
    data_length = len(data_matrix_checked[0])
    step = data_length/n_trials
    print step

    bad_electrodes = []

    for electrode in range(num_electrodes):
        for k in range(0, data_length, step):
            data = data_matrix_checked[electrode, k:k+step]
            inter_trial_step = 100  # 200ms step to check for artifacts
            for i in range(0, step, inter_trial_step):
                inter_trial_data = data[i:i+inter_trial_step]
                minimum = min(inter_trial_data)
                maximum = max(inter_trial_data)
                if abs(maximum-minimum) >= 100.0:
                    data_matrix_checked[electrode, :] *= 0.0
                    bad_electrodes.append(electrode)
                    print "Large variations detected at trial=" + str((int(k/step))+1) + ", time=" + str(2*i) + " ms"
                    # print "Electrode #" + str(electrode+1) + " - Max-Min=>" + str(maximum-minimum)

            std = np.std(data)
            if std > threshold:
                data_matrix_checked[electrode, :] *= 0.0
                bad_electrodes.append(electrode)
                print "Bad electrode detected (std)."
                print "Electrode #" + str(electrode+1) + " - Std=" + str(std)

            d1 = np.r_[0, np.abs(data[1:] - data[:-1])]
            d2 = np.r_[np.abs(data[1:] - data[:-1]), 0]
            bad = (d1 > threshold/2.0) | (d2 > threshold/2.0)
            if np.sum(bad):
                data_matrix_checked[electrode, :] *= 0.0
                bad_electrodes.append(electrode)
                print "Bad electrode detected (diff amp)."
                print "Electrode #" + str(electrode+1) + " - deltaA>" + str(threshold/2.0)

    return data_matrix_checked, bad_electrodes


def check_for_bad_trials(data_matrix, threshold=40.0, n_trials=210, inter_trial_step=100):

    data_matrix_checked = data_matrix

    num_electrodes = len(data_matrix_checked)
    data_length = len(data_matrix_checked[0])

    step = data_length/n_trials

    bad_trials = []
    bad_trials_matrix = np.zeros((num_electrodes, n_trials), np.int)

    for k in range(0, data_length, step):
        trial = int(k/step) + 1
        bad_electrodes = []
        detected = False
        for electrode in range(num_electrodes):
            data = data_matrix_checked[electrode, k+250:k+step-500]
            for i in range(0, 2249, inter_trial_step):

                inter_trial_data = data[i:i+inter_trial_step]

                minimum = min(inter_trial_data)
                maximum = max(inter_trial_data)

                d1 = np.r_[0, np.abs(inter_trial_data[1:] - inter_trial_data[:-1])]
                d2 = np.r_[np.abs(inter_trial_data[1:] - inter_trial_data[:-1]), 0]
                b = (d1 > 0.5*threshold) | (d2 > 0.5*threshold)
                if np.sum(b):
                    detected = True
                    # print "Detected by gradient."

                if abs(maximum-minimum) >= threshold:
                    detected = True
                    if electrode not in bad_electrodes:
                        bad_trials_matrix[electrode, trial-1] = 1  # zero-indexed matrix
                        bad_electrodes.append(electrode)

        if detected:
            if trial not in bad_trials:
                bad_trials.append(trial)
                n_bad_electrodes = len(bad_electrodes)
                print "Large fluctuations detected in " + str(n_bad_electrodes) + " electrode(s) at trial=" + \
                      str(trial)

    return data_matrix_checked, bad_trials, bad_trials_matrix


def check_for_bad_electrodes_based_on_impedance(electrodes_array, impedances_array, impedance_limit=100.0):

    num_electrodes = len(electrodes_array)

    for i in range(num_electrodes):
        electrode = electrodes_array[i] - 1
        impedance = float(impedances_array[electrode][0])
        if abs(impedance) > 100.0:
            # This might not work. Use 'electrodes_array = np.delete(....) '
            np.delete(electrodes_array, i)
            num_electrodes -= 1
            print "Bad electrode detected."
            print "Electrode #" + str(electrode+1) + " - Impedance: " + str(impedance)

    return electrodes_array


def get_amplitude_modulation(data):
    n = len(data)
    absolute_value = np.fabs(data)
    x = signal.argrelextrema(absolute_value, np.greater)[0]
    y = absolute_value[x]
    amplitude_modulation = interpolate.PchipInterpolator(x, y)(np.arange(n))
    return amplitude_modulation


