from __future__ import print_function
import time
from sklearn import svm
from sklearn.tree import DecisionTreeClassifier
from sklearn.neighbors import KNeighborsClassifier
from sklearn.ensemble import RandomForestClassifier
from sklearn.ensemble import AdaBoostClassifier
from sklearn.naive_bayes import GaussianNB
from sklearn.lda import LDA
from sklearn import linear_model
from sklearn import metrics
from sklearn.neural_network import BernoulliRBM
from sklearn.pipeline import Pipeline
from sklearn.grid_search import GridSearchCV
from sklearn import cross_validation


def grid_search(classifier_name, trained_data=(), feature_selector=(), parameters={}):
    start_time = time.time()
    trained_set = trained_data[0]
    trained_classes = trained_data[1]
    classifier = svm.SVC(kernel=classifier_name)

    if feature_selector:
        classifier = Pipeline([feature_selector, (classifier_name, classifier)])

    classifier = GridSearchCV(classifier, parameters)
    classifier.fit(trained_set, trained_classes)
    best_parameters = classifier.best_estimator_.get_params()
    params_keys = parameters.keys()
    # print("\n########## Grid Search Results ##########")
    print("Best Gamma: ", best_parameters[params_keys[1]])
    print("Best C: ", best_parameters[params_keys[0]])
    # print("Searching Time: ", time.time() - start_time)

    return classifier, best_parameters


def predict_with_cross_validation(classifier, clf_name, trained_data=(), test_data=()):
    start_time = time.time()
    trained_set = trained_data[0]
    trained_classes = trained_data[1]
    test_set = test_data[0]
    test_classes = test_data[1]
    scores = cross_validation.cross_val_score(classifier, trained_set, trained_classes, cv=5)
    predicted = cross_validation.cross_val_predict(classifier, test_set, test_classes, cv=5)

    print("\n########## Cross-validation Results ##########")
    print("Classifier: ", clf_name)
    print("Accuracy: %0.2f (+/- %0.2f)" % (scores.mean(), scores.std() * 2))
    print("Accuracy score: ", metrics.accuracy_score(test_classes, predicted))
    print("Process time: ", time.time() - start_time)
    print("\n")


def predict_test_data(classifier, clf_name, test_data=(), feature_selector=()):
    start_time = time.time()
    test_set = test_data[0]
    test_classes = test_data[1]
    score = classifier.score(test_set, test_classes)
    predicted = classifier.predict(test_set)

    if feature_selector:
        selector_name = feature_selector[0]
        selected_features = classifier.best_estimator_.named_steps[selector_name].get_support()
    else:
        selected_features = ''

    print_scores(clf_name, selected_features, score, test_classes, predicted, start_time)


def map_classifier_name(classifier_name):
    if classifier_name == 'rbf' or classifier_name == 'linear' or classifier_name == 'poly' or classifier_name == 'sigmoid':
        return svm.SVC(kernel=classifier_name)
    elif classifier_name == 'nearest_neighbors':
        return KNeighborsClassifier()
    elif classifier_name == 'decision_tree':
        return DecisionTreeClassifier()
    elif classifier_name == 'random_forest':
        return RandomForestClassifier()
    elif classifier_name == 'ada_boost':
        return AdaBoostClassifier()
    elif classifier_name == 'naive_bayes':
        return GaussianNB()
    elif classifier_name == 'lda':
        return LDA()


def linear_svm(trained_set, test_set, trained_classes, test_classes, feature_selector=()):
    start_time = time.time()
    name = "Linear SVM"
    feature_selector_name = ""
    classifier = svm.SVC(kernel="linear")

    if feature_selector:
        classifier = Pipeline([feature_selector, (name, svm.SVC(kernel="linear"))])
        feature_selector_name = feature_selector[0]

    train_and_predict_data(name, feature_selector_name, classifier, trained_set, test_set, trained_classes, test_classes, start_time)

    return classifier


def rbf_svm(trained_set, test_set, trained_classes, test_classes, feature_selector=(), parameters={}):
    start_time = time.time()
    name = "RBF SVM"
    feature_selector_name = ""
    classifier = svm.SVC(parameters)

    if feature_selector:
        classifier = Pipeline([feature_selector, (name, classifier)])
        feature_selector_name = feature_selector[0]

    train_and_predict_data(name, feature_selector_name, classifier, trained_set, test_set, trained_classes, test_classes, start_time)


def polynomial_svm(trained_set, test_set, trained_classes, test_classes, feature_selector=()):
    start_time = time.time()
    name = "Polynomial"
    feature_selector_name = ""
    classifier = svm.SVC(kernel="poly")

    if feature_selector:
        classifier = Pipeline([feature_selector, (name, classifier)])
        feature_selector_name = feature_selector[0]

    train_and_predict_data(name, feature_selector_name, classifier, trained_set, test_set, trained_classes, test_classes, start_time)


def sigmoid_svm(trained_set, test_set, trained_classes, test_classes, feature_selector=()):
    start_time = time.time()
    name = "Sigmoid"
    feature_selector_name = ""
    classifier = svm.SVC(kernel="sigmoid")

    if feature_selector:
        classifier = Pipeline([feature_selector, (name, svm.SVC(kernel="sigmoid"))])
        feature_selector_name = feature_selector[0]

    train_and_predict_data(name, feature_selector_name, classifier, trained_set, test_set, trained_classes, test_classes, start_time)


def nearest_neighbors(trained_set, test_set, trained_classes, test_classes, feature_selector=()):
    start_time = time.time()
    name = "Nearest Neighbors"
    feature_selector_name = ""
    classifier = KNeighborsClassifier()

    if feature_selector:
        classifier = Pipeline([feature_selector, (name, KNeighborsClassifier(3))])
        feature_selector_name = feature_selector[0]

    train_and_predict_data(name, feature_selector_name, classifier, trained_set, test_set, trained_classes, test_classes, start_time)


def decision_tree(trained_set, test_set, trained_classes, test_classes, feature_selector=()):
    start_time = time.time()
    name = "Decision Tree"
    feature_selector_name = ""
    classifier = DecisionTreeClassifier()

    if feature_selector:
        classifier = Pipeline([feature_selector, (name, DecisionTreeClassifier())])
        feature_selector_name = feature_selector[0]

    train_and_predict_data(name, feature_selector_name, classifier, trained_set, test_set, trained_classes, test_classes, start_time)


def random_forest(trained_set, test_set, trained_classes, test_classes, feature_selector=()):
    start_time = time.time()
    name = "Random Forest"
    feature_selector_name = ""
    classifier = RandomForestClassifier()

    if feature_selector:
        classifier = Pipeline([feature_selector, (name, RandomForestClassifier())])
        feature_selector_name = feature_selector[0]

    train_and_predict_data(name, feature_selector_name ,classifier, trained_set, test_set, trained_classes, test_classes, start_time)


def adaboost(trained_set, test_set, trained_classes, test_classes, feature_selector=()):
    start_time = time.time()
    name = "AdaBoost"
    feature_selector_name = ""
    classifier = AdaBoostClassifier()

    if feature_selector:
        classifier = Pipeline([feature_selector, (name, AdaBoostClassifier())])
        feature_selector_name = feature_selector[0]

    train_and_predict_data(name, feature_selector_name, classifier, trained_set, test_set, trained_classes, test_classes, start_time)


def naive_bayes(trained_set, test_set, trained_classes, test_classes, feature_selector=()):
    start_time = time.time()
    name = "Gaussian Naive Bayes"
    feature_selector_name = ""
    classifier = GaussianNB()

    if feature_selector:
        classifier = Pipeline([feature_selector, (name, GaussianNB())])
        feature_selector_name = feature_selector[0]

    train_and_predict_data(name, feature_selector_name, classifier, trained_set, test_set, trained_classes, test_classes, start_time)


def linear_discriminant_analyze(trained_set, test_set, trained_classes, test_classes, feature_selector=()):
    start_time = time.time()
    name = "Linear Discriminant Analyze"
    feature_selector_name = ""
    classifier = LDA()

    if feature_selector:
        classifier = Pipeline([feature_selector, (name, LDA())])
        feature_selector_name = feature_selector[0]

    train_and_predict_data(name, feature_selector_name, classifier, trained_set, test_set, trained_classes, test_classes, start_time)


def artificial_neural_network(trained_set, test_set, trained_classes, test_classes):
    start_time = time.time()
    logistic = linear_model.LogisticRegression()
    rbm = BernoulliRBM()
    rbm.n_components = 100
    classifier = Pipeline(steps=[('rbm', rbm), ('logistic', logistic)])
    classifier.fit(trained_set, trained_classes)
    logistic_classifier = linear_model.LogisticRegression(C=100.0)
    logistic_classifier.fit(trained_set, trained_classes)
    predicted = classifier.predict(test_set)
    print("Method: ", "Restricted Boltzmann Machine")
    print("Logistic regression using RBM features:\n%s\n" % (metrics.classification_report(
        test_classes,
        predicted
    )))
    print("Score: ", classifier.score(test_set, test_classes))
    print("--- %s seconds ---" % (time.time() - start_time))
    print("\n")


def train_and_predict_data(name, feature_selector_name, classifier, trained_set, test_set, trained_classes, test_classes, start_time):
    classifier.fit(trained_set, trained_classes)
    score = classifier.score(test_set, test_classes)
    predicted = classifier.predict(test_set)

    if feature_selector_name:
        selected_features = classifier.named_steps[feature_selector_name].get_support()
    else:
        selected_features = "No Feature Selector"

    print_scores(name, selected_features, score, test_classes, predicted, start_time)


def print_scores(name, selected_features, score, test_classes, predicted, start_time):
    # print("########## Optimized Classifier Results ##########")
    print("Classifier: ", name)
    # print("Score: %.2f %% " % (score*100))
    print("True:", end=" ")
    [print(int(test_classes[i]), end=" ") for i in range(len(test_classes))]
    print("\nPred:", end=" ")
    [print(int(predicted[i]), end=" ") for i in range(len(predicted))]
    # print("\nSelected Features: ", selected_features)
    print("\nProcess time: %s seconds" % (time.time() - start_time))
    introduction = "Score: %.2f %% " % (score*100)
    print("%s\n%s\n%s" % ('-' * len(introduction), introduction, '-' * len(introduction)))
    print("\n")
