from scipy.signal import butter, lfilter, filtfilt
import numpy as np
import matplotlib.pyplot as plt
from scipy.signal import freqz
import sys
import constants


def notch_filter(data, cutoff=50.0, width=2.0, fs=500.0):
    # create the 50 Hz notch filter
    bp2_stop_Hz = np.array([cutoff-(width/2), cutoff+(width/2)])

    b2, a2 = butter(2, bp2_stop_Hz/(fs / 2.0), 'bandstop')

    y = filtfilt(b2, a2, data)
    return y


def laplacian_spatial_filter(data_matrix, type_size='small'):

    data_length = len(data_matrix[0])
    n_electrodes = len(data_matrix)

    c3 = constants.c3_electrode[0]
    c4 = constants.c4_electrode[0]
    cz = constants.cz_electrode[0]

    if n_electrodes < 256:
        c3_index = constants.electrodes_motor_cortex.index(c3)
        c4_index = constants.electrodes_motor_cortex.index(c4)
        cz_index = constants.electrodes_motor_cortex.index(cz)
        c3_data = data_matrix[c3_index]
        c4_data = data_matrix[c4_index]
        cz_data = data_matrix[cz_index]
    else:
        c3_data = data_matrix[c3]
        c4_data = data_matrix[c4]
        cz_data = data_matrix[cz]

    sum_neighbors_c3 = np.zeros(data_length, np.float)
    sum_neighbors_c4 = np.zeros(data_length, np.float)
    sum_neighbors_cz = np.zeros(data_length, np.float)

    # Spatial filtered Cz electrode
    neighbors_cz = constants.neighbors_cz
    n_neighbors_cz = len(neighbors_cz)
    for k in range(n_neighbors_cz):
            if n_electrodes < 256:
                cz_neighbor_index = constants.electrodes_motor_cortex.index(neighbors_cz[k])
                sum_neighbors_cz += data_matrix[cz_neighbor_index]
            else:
                sum_neighbors_cz = np.add(sum_neighbors_cz, data_matrix[neighbors_cz[k]])

    # Small laplacian:
    if type_size == 'small':

        neighbors_c3 = constants.neighbors_c3_small
        neighbors_c4 = constants.neighbors_c4_small
        n_neighbors = len(neighbors_c3)

        for k in range(n_neighbors):
            if n_electrodes < 256:
                c3_neighbor_index = constants.electrodes_motor_cortex.index(neighbors_c3[k])
                sum_neighbors_c3 += data_matrix[c3_neighbor_index]

                c4_neighbor_index = constants.electrodes_motor_cortex.index(neighbors_c4[k])
                sum_neighbors_c4 += data_matrix[c4_neighbor_index]
            else:
                sum_neighbors_c3 = np.add(sum_neighbors_c3, data_matrix[neighbors_c3[k]])
                sum_neighbors_c4 = np.add(sum_neighbors_c4, data_matrix[neighbors_c4[k]])

    # Large laplacian:
    else:

        neighbors_c3 = constants.neighbors_c3_large
        neighbors_c4 = constants.neighbors_c4_large
        n_neighbors = len(neighbors_c3)

        for k in range(n_neighbors):
            if n_electrodes < 256:
                c3_neighbor_index = constants.electrodes_motor_cortex.index(neighbors_c3[k])
                sum_neighbors_c3 += data_matrix[c3_neighbor_index]

                c4_neighbor_index = constants.electrodes_motor_cortex.index(neighbors_c4[k])
                sum_neighbors_c4 += data_matrix[c4_neighbor_index]
            else:
                sum_neighbors_c3 = np.add(sum_neighbors_c3, data_matrix[neighbors_c3[k]])
                sum_neighbors_c4 = np.add(sum_neighbors_c4, data_matrix[neighbors_c4[k]])

    c3_spatial_filtered = np.subtract(c3_data, np.multiply(1.0/float(n_neighbors), sum_neighbors_c3))
    c4_spatial_filtered = np.subtract(c4_data, np.multiply(1.0/float(n_neighbors), sum_neighbors_c4))
    cz_spatial_filtered = np.subtract(cz_data, np.multiply(1.0/float(n_neighbors_cz), sum_neighbors_cz))

    return c3_spatial_filtered, c4_spatial_filtered, cz_spatial_filtered


def butter_bandpass(low_cut, high_cut, fs=500.0, order=8):
    nyq = 0.5 * fs
    low = low_cut / nyq
    high = high_cut / nyq
    b, a = butter(order, [low, high], btype='band')
    return b, a


def butter_bandpass_filter(data, low_cut, high_cut, fs=500.0, order=4):
    b, a = butter_bandpass(low_cut, high_cut, fs=int(fs), order=order)
    y = filtfilt(b, a, data)
    # y = lfilter(b, a, data)
    return y


def test_filter_response():
    # Sample rate and desired cutoff frequencies (in Hz).
    fs = 500.0
    lowcut = 1.6
    highcut = 60.0

    # Plot the frequency response for a few different orders.
    plt.figure(1)
    plt.clf()
    for order in [3, 4, 5, 6, 8, 9]:
        b, a = butter_bandpass(lowcut, highcut, fs, order=order)
        w, h = freqz(b, a)
        plt.plot((fs * 0.5 / np.pi) * w, abs(h), label="order = %d" % order)

    plt.plot([0, 0.5 * fs], [np.sqrt(0.5), np.sqrt(0.5)],
             '--', label='sqrt(0.5)')
    plt.xlabel('Frequency (Hz)')
    plt.ylabel('Gain')
    plt.grid(True)
    plt.legend(loc='best')

    try:
        plt.show(block=True)
    except KeyboardInterrupt, ki:
        sys.exit()


















